#include "main.h"
#include "globals.h"
#include "supfiles.h"
#include "aosinfo.h"

static BYTE genpal[32];
static BYTE bigpal[1024*3];


extern int throttle;
extern int vol;
extern int record;
extern int playback;
extern int soundcard;
extern int samplerate;
extern int background;
extern int arcade;
extern int precalculate;
extern char rompath[512];
extern char samplepath[512];
extern char snappath[512];
extern int vsync;
int origrotate=0;
extern int scrolling;
extern int scale;
extern int crop;
extern int VertScan;

//command line switches
static CONFIGITEM configs[]=
{
   {&VertScan,"Vertical Scanlines\n#Using correct scanlines for vertical games running on a horizontal monitor","vertscan",0,1,0,0},
   {&UsingVESA,"Use VESA video modes (if available)","vesa",0,1,1,0},
   {&vsync,"Sync to video display rate","vsync",0,1,0,0},
   {&ror,"Rotate Right","ror",0,1,0,0},
   {&rol,"Rotate Left","rol",&ror,1,0,0},
   {&precalculate,"Generate Precalculated graphic files","precalc",0,1,0,0},
   {&flipxy,"Flip Screen","flipxy",0,0,0,0},
   {&scanlines,"Scanline video modes","scanlines",0,1,0,0},
   {&arcade,"Arcade Monitor Mode","arcade",&scanlines,1,0,0},
   {&throttle,"Throttle game speed","throttle",0,1,1,0},
   {&record,"Record Input","record",0,0,0,0},
   {&playback,"Playback Input","playback",&record,0,0,0},
   {&background,"Background graphics","background",0,1,1,0},
   {&logging,"Generate VANTAGE.LOG","log",0,0,0,0},
   {&origrotate,"Run game at original rotation","origrotate",0,1,0,0},
   {&scale,"Scales tall vertical games to fit screen","scale",&scrolling,1,0,0},
   {&crop,"Crops  tall vertical games to fit screen","crop",&scrolling,1,0,0},
   {0,"","",0,0,0,0}
};

//command line numeric values
static CONFIGITEM configvals[]=
{
   {&centerx,"Centre Screen (horizontally)","centerx",0,1,0,0},
   {&centery,"Centre Screen (vertically)","centery",0,1,0,0},
   {&vol,"Volume level 0-15","vol",0,1,15,0},
   {&joytype,"Joystick type\n#0=none\n#1=2 button joystick/pad\n#2=4 button joystick/pad\n"\
               "#3=6 button joystick/pad\n#4=8 button joystick/pad\n"\
               "#5=sidewinder\n#6=GamePad Pro\n#7=PSX pad","joy",0,1,0,0},
   {&soundcard,"Type of soundcard to use\n"
               "#0=Silence\n"
               "#1=Sound Blaster\n"
               "#2=Sound Blaster AWE32\n"
               "#3=Pro Audio Spectrum\n"
               "#4=Ultrasound Max (CS4231 Codec)\n"
               "#5=Ultrasound\n"
               "#6=Windows Sound System\n"
               "#7=Ensoniq Soundscape","soundcard",0,1,0,0},
   {&samplerate,"Sample Rate to use (11025,22050,44100)","samplerate",0,1,22050,0},
   {&frameskip,"frame skip (0 - 11)","frameskip",0,1,0,0},
   {0,"","",0,0,0,0}
};

//command line text values
static TEXTITEM textvals[]=
{
   {rompath,"Location of ROM sets","rompath"},
   {samplepath,"Location of Samples","samples"},
   {snappath,"Location of Snap Shots","snap"},
   {0,"",""}
};
//list of supported boards
static BOARD boards[]=
{

   {"triplep",
      "Triple Punch",
      "triplep",
      1,
      {
         {51200,ScrambleRead,TriplePWrite,TriplePReadPorts,TriplePWritePorts,scramint,1}
      },
      {
         {triplepcode,triplepgfx,triplepcol,0}
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      triplepinputs,
      0,
      initscram,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scramupdatescreen,
      TRIPLEPDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      0,
      0,
      1,
      inittriplepsnd,
      AY8910_sh_update,0,0
   },
   {"scramble",   //Konami
      "Scramble",
      "scramble",
      2,
      {
         {51200,ScrambleRead,ScrambleWrite,ReadPorts,WritePorts,scramint,1},
         {29829,ScrambleSndRead,ScrambleSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {scramcode,scramgfx,scramcol,0},
         {scramsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      scraminputs,
      0,
      initscram,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scramupdatescreen,
      SCRAMDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"atlantis",
      "Battle of Atlantis (set 1)",
      "atlantis",
      2,
      {
         {51200,AtlantisRead,ScrambleWrite,ReadPorts,WritePorts,scramint,1},
         {29829,ScrambleSndRead,ScrambleSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {atlantiscode,atlantisgfx,atlantiscol,0},
         {atlantissndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      atlantisinputs,
      0,
      initscram,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scramupdatescreen,
      ATLANTISDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,
      0,0
   },
   {"devilfsh",   //Konami
      "Devil Fish",
      "devilfsh",
      2,
      {
         {51200,DevilFishRead,DevilFishWrite,ReadPorts,WritePorts,devilint,1},
         {29829,ScrambleSndRead,ScrambleSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {devilfishcode,devilfishgfx,devilfishcol,0},
         {devilfishsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      devilfishinputs,
      0,
      initdevilfish,cleargal,
      0,
      devilfishgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scramupdatescreen,
      DEVLIFISHDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"scobra",   //Konami
      "Super Cobra",
      "scobra",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {scobracode,scobragfx,scobracol,0},
         {scobrasndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      scobrainputs,
      0,
      initscobra,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      SCOBRADIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"superbon",
      "Super Bond",
      "superbon",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {superboncode,superbongfx,superboncol,0},
         {superbonsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      superboninputs,
      0,
      initsuperbon,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      SUPERBONDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"spdcoin",   //Konami
      "Speed Coin (prototype)",
      "spdcoin",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {spdcoincode,spdcoingfx,spdcoincol,0},
         {spdcoinsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      spdcoininputs,
      0,
      initscobra,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      SPDCOINDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,
      0,0
   },
   {"anteater",   //Konami
      "Anteater",
      "anteater",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {anteatercode,anteatergfx,anteatercol,0},
         {anteatersndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      anteaterinputs,
      0,
      initscobra,cleargal,
      anteaterdecodegfx,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      ANTEATERDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initanteatersnd,
      AY8910_sh_update,0,0
   },
   {"losttomb",   //Konami
      "Lost Tomb (easy)",
      "losttomb",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {losttombcode,losttombgfx,losttombcol,0},
         {losttombsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      losttombinputs,
      0,
      initscobra,cleargal,
      losttombdecodegfx,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      LOSTTOMBDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"hustler",   //Konami
      "Video Hustler",
      "hustler",
      2,
      {
         {51200,HustlerRead,HustlerWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,HustlerSndReadPorts,HustlerSndWritePorts,0,1}
      },
      {
         {hustlercode,hustlergfx,hustlercol,0},
         {hustlersndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      hustlerinputs,
      0,
      inithustler,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      hustlerupdatescreen,
      HUSTLERDIPS, //dip switch settings
      0,0,0,1,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initfrogsnd,
      AY8910_sh_update,0,0
   },
   {"tazmania",
      "Tazz-Mania (Scramble hardware)",
      "tazmania",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {tazcode,tazgfx,tazcol,0},
         {tazsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      tazinputs,
      0,
      initscobra,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      TAZDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,1,0
   },
   {"armorcar",
      "Armored Car (set 1)",
      "armorcar",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {armorcode,armorgfx,armorcol,0},
         {armorsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      armorinputs,
      0,
      initscobra,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scobraupdatescreen,
      ARMORDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,1,0
   },
   {"calipso",
      "Calipso",
      "calipso",
      2,
      {
         {51200,SCobraRead,SCobraWrite,ReadPorts,WritePorts,scobraint,1},
         {29829,SCobraSndRead,SCobraSndWrite,SCobraSndReadPorts,SCobraSndWritePorts,0,1}
      },
      {
         {calipsocode,calipsogfx,calipsocol,0},
         {calipsosndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      calipsoinputs,
      0,
      initscobra,cleargal,
      0,
      calipsogfxs,
      1024,  //number of characters
      16,   //number of character cols
      256,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      calipsoupdatescreen,
      CALIPSODIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initcalipsosnd,
      AY8910_sh_update,0,0
   },

   {"theend",   //Konami
      "The End",
      "theend",
      2,
      {
         {51200,ScrambleRead,ScrambleWrite,ReadPorts,WritePorts,scramint,1},
         {29829,ScrambleSndRead,ScrambleSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {theendcode,theendgfx,theendcol,0},
         {theendsndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      scraminputs,
      0,
      initscram,cleargal,
      0,
      galgfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,scrampalupd,0,
      scramupdatescreen,
      SCRAMDIPS, //dip switch settings
      0,0,1,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      1,
      initscramsnd,
      AY8910_sh_update,
      0,0
   },
   {"amidar",  //Konami
      "Amidar",
      "amidar",
      2,
      {
         {51200,AmidarRead,AmidarWrite,ReadPorts,WritePorts,amidarint,1},
         {29829,AmidarSndRead,AmidarSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {amidarcode,amidargfx,amidarcol,0},
         {amidarsndcode,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      amidarinputs,
      0,
      initamidar,clearamidar,
      0,
      amdgfxs,
      256,
      16,
      64,
      16,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      setpal,0,0,
      amdupdatescreen,
      AMIDARDIPS,
      0,0,0,1,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      2,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"amidaru",  //Stern License (Konami)
      "Amidar US",
      "amidar",
      2,
      {
         {51200,AmidarRead,AmidarWrite,ReadPorts,WritePorts,amidarint,1},
         {29829,AmidarSndRead,AmidarSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {amidarcodeu,amidargfxu,amidarcol,0},
         {amidarsndcode,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      amidarinputs,
      0,
      initamidar,clearamidar,
      0,
      amdgfxs,
      256,
      16,
      64,
      16,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      setpal,0,0,
      amdupdatescreen,
      AMIDARDIPS,
      0,0,0,1,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      2,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"turtles",  //Konami
      "Turtles",
      "turtles",
      2,
      {
         {51200,AmidarRead,AmidarWrite,ReadPorts,WritePorts,amidarint,1},
         {29829,AmidarSndRead,AmidarSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {turtlescode,turtlesgfx,turtlescol,0},
         {turtlessndcode,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,23,0,16,0,3},{256,224,0,20,0,16,0}},
      amidarinputs,
      0,
      initamidar,clearamidar,
      0,
      amdgfxs,
      256,
      16,
      64,
      16,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      setpal,0,0,
      amdupdatescreen,
      TURTLESDIPS,
      0,0,0,1,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      2,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"600",  //Konami
      "600",
      "turtles",
      2,
      {
         {51200,AmidarRead,AmidarWrite,ReadPorts,WritePorts,amidarint,1},
         {29829,AmidarSndRead,AmidarSndWrite,ScrambleSndReadPorts,ScrambleSndWritePorts,0,1}
      },
      {
         {_600code,_600gfx,turtlescol,0},
         {_600sndcode,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,20,0,16,0}},
      amidarinputs,
      0,
      initamidar,clearamidar,
      0,
      amdgfxs,
      256,
      16,
      64,
      16,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      setpal,0,0,
      amdupdatescreen,
      TURTLESDIPS,
      0,0,0,1,0,ORIG_ROT_LEFT,
      scrampre,
      0,
      2,
      initscramsnd,
      AY8910_sh_update,0,0
   },
   {"galaga",  //NAMCO
      "Galaga",
      "galaga",
      3,
      {
         {52083,GalagaRead,GalagaWrite,ReadPorts,WritePorts,galagaint,GALAGA_NMI_SLICE},
         {52083,GalagaRead2,GalagaWrite2,ReadPorts,WritePorts,galagaint2,1},
         {52083,GalagaRead3,GalagaWrite3,ReadPorts,WritePorts,galagaint3,2}
      },
      {
         {galagacode,galagagfx,galagacol,galagasound},
         {galagacode2,0,0,0},
         {galagacode3,0,0,0}
      },
      "galaga",
      {{"bang.wav"}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      galagainputs,
      0,
      initgalaga,cleargalaga,
      0,
      galagagfxs,
      128,
      32,
      128,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      galagasetpal,0,0,
      galagaupdatescreen,
      GALAGADIPS,
      0,0,0,0,1,ORIG_ROT_LEFT,
      galagapre,galagapost,
      4000,
      namcostartsound,
      namcoupdatesound,0,1
   },
   {"galagads",  //hack
      "Galaga Fast Shot Hack",
      "galaga",
      3,
      {
         {52083,GalagaRead,GalagaWrite,ReadPorts,WritePorts,galagaint,GALAGA_NMI_SLICE},
         {52083,GalagaRead2,GalagaWrite2,ReadPorts,WritePorts,galagaint2,1},
         {52083,GalagaRead3,GalagaWrite3,ReadPorts,WritePorts,galagaint3,2}
      },
      {
         {galagadscode,galagagfx,galagacol,galagasound},
         {galagadscode2,0,0,0},
         {galagadscode3,0,0,0}
      },
      "galaga",
      {{"bang.wav"}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      galagainputs,
      0,
      initgalaga,cleargalaga,
      0,
      galagagfxs,
      128,
      32,
      128,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      galagasetpal,0,0,
      galagaupdatescreen,
      GALAGADIPS,
      0,0,0,0,1,ORIG_ROT_LEFT,
      galagapre,galagapost,
      4000,
      namcostartsound,
      namcoupdatesound,0,1
   },
   {"galaga84",  //hack
      "Galaga 84",
      "galaga",
      3,
      {
         {52083,GalagaRead,GalagaWrite,ReadPorts,WritePorts,galagaint,GALAGA_NMI_SLICE},
         {52083,GalagaRead2,GalagaWrite2,ReadPorts,WritePorts,galagaint2,1},
         {52083,GalagaRead3,GalagaWrite3,ReadPorts,WritePorts,galagaint3,2}
      },
      {
         {galaga84code,galaga84gfx,galagacol,galagasound},
         {galaga84code2,0,0,0},
         {galagacode3,0,0,0}
      },
      "galaga",
      {{"bang.wav"}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      galagainputs,
      0,
      initgalaga,cleargalaga,
      0,
      galagagfxs,
      128,
      32,
      128,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      galagasetpal,0,0,
      galagaupdatescreen,
      GALAGADIPS,
      0,0,0,0,1,ORIG_ROT_LEFT,
      galagapre,galagapost,
      4000,
      namcostartsound,
      namcoupdatesound,0,1
   },
   {"pacman",  //NAMCO
      "Pac-Man",
      "pacman",
      1,
      {{51200,PacRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{paccode,pacgfx,paccol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      PACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"theglob",
      "The Glob",
      "theglob",
      1,
      {{51200,TheGlobRead,TheGlobWrite,TheGlobReadPorts,WritePorts,pacint,1}},
      {{theglobcode,theglobgfx,theglobcol,theglobsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      theglobinputs,
      0,
      inittheglob,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      THEGLOBDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,
      namcostartsound,
      namcoupdatesound,
      0,0
   },
   {"crush2",
      "Crush Roller (Kural Esco - bootleg?)",
      "crush",
      1,
      {{51200,CrushRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{crush2code,crush2gfx,crush2col,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      crushinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      CRUSHDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"vanvan",
      "Van Van Car",
      "vanvan",
      1,
      {{51200,VanVanRead,PacWrite,ReadPorts,VanVanWritePorts,vanvanint,1}},
      {{vanvancode,vanvangfx,vanvancol,0}},
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      vanvaninputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      vanvandatescreen,
      VANVANDIPS,
      0,0,0,1,1,ORIG_ROT_RIGHT,0,0,
      1,
      initvanvansnd, //start sound
      SN76496_sh_update, //update sound
      0,0
   },
   {"pacplus",  //NAMCO
      "Pac-Man Plus",
      "pacplus",
      1,
      {{51200,PacRead,PacWrite,ReadPorts,PacWritePorts,pacint,1}},
      {{pacpluscode,pacplusgfx,pacpluscol,pacplussnd}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpacplus,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      PACNONSPEEDDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"eyes",
      "Eyes",
      "eyes",
      1,
      {{51200,LizWizRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{eyescode,eyesgfx,eyescol,eyessnd}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      lizwizinputs,
      0,
      initeyes,clearpac,
      eyesdecodegfx,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      EYESDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"mrtnt",
      "Mr TNT",
      "mrtnt",
      1,
      {{51200,LizWizRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{mrtntcode,mrtntgfx,mrtntcol,mrtntsnd}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      lizwizinputs,
      0,
      initeyes,clearpac,
      eyesdecodegfx,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      EYESDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"lizwiz",  //NAMCO
      "Lizard Wizard",
      "lizwiz",
      1,
      {{51200,LizWizRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{lizwizcode,lizwizgfx,lizwizcol,lizwizsnd}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      lizwizinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      LIZWIZDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,1,0
   },
   {"pengo2u",  //SEGA
      "Pengo Set 2 Unencrypted",
      "pengo",
      1,
      {{51200,PengoURead,PengoUWrite,ReadPorts,WritePorts,pengoint,1}},
      {{pengo2ucode,pengogfx,pengocol,pengosnd}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pengoinputs,
      0,
      initpengo2u,clearpac,
      0,
      pengogfxs,
      512,
      32,
      128,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pengosetpal,0,0,
      pacupdatescreen,
      PENGODIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"piranha",  //NAMCO
      "Piranha",
      "pacman",
      1,
      {{51200,PacRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{piranacode,piranagfx,piranacol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      PACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"hangly",  //hack
      "Hangly Man",
      "pacman",
      1,
      {{51200,PacRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{hanglycode,pacgfx,paccol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      PACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,
      namcostartsound,namcoupdatesound,0,0
   },
   {"mspacman",   //MIDWAY (MW (C) but really a bootleg)
      "Ms Pac-Man",
      "mspacman",
      1,
      {{51200,PacRead,MsPacWrite,ReadPorts,WritePorts,pacint,1}},
      {{mspaccode,mspacgfx,mspaccol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      MSPACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"mspacatk",
      "Ms Pac-Man Plus",
      "mspacman",
      1,
      {{51200,PacRead,MsPacWrite,ReadPorts,WritePorts,pacint,1}},
      {{mpacacode,mspacgfx,mspaccol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      MSPACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"puckman",  //??? - hack
      "PuckMan",
      "pacman",
      1,
      {{51200,PacRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{puckcode,puckgfx,puckcol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      PACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   {"pacheart",  //??? - hack
      "Pac-Man (Hearts)",
      "pacman",
      1,
      {{51200,PacRead,PacWrite,ReadPorts,WritePorts,pacint,1}},
      {{pacheartcode,pacheartgfx,puckcol,pacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      pacinputs,
      0,
      initpac,clearpac,
      0,
      pacgfxs,
      256,
      32,
      64,
      32,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      pacsetpal,0,0,
      pacupdatescreen,
      PACDIPS,
      0,0,0,1,1,ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   },
   //NAMCO
   {"galaxian",      //Game Name
      "Galaxians",   //Description
      "galaxian",    //parent
      1,             //no. CPUs
      {{51200,GameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{galaxcode,galaxgfx,galaxcol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      GALDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,  //mutually exclusive inputs
      0  // scaler
   },
   {"redufo",      //Game Name
      "Red UFO",   //Description
      "galaxian",    //parent
      1,             //no. CPUs
      {{51200,GameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{redufocode,redufogfx,redufocol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      REDUFODIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,  //mutually exclusive inputs
      0
   },
   {"warofbug",      //Game Name
      "War of the Bugs",   //Description
      "warofbug",    //parent
      1,             //no. CPUs
      {{51200,GameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{warofbugcode,warofbuggfx,warofbugcol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      wobinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      WOBDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"pacmanbl",      //Game Name
      "Pac-Man Bootleg",   //Description
      "pacman",    //parent
      1,             //no. CPUs
      {{51200,PacblGameRead,PacblGameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{pacblcode,pacblgfx,pacblcol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      pacblinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      pacblgfxs, //gfx decode structure
      256,  //number of characters
      32,   //number of character cols
      64,   //number of sprites
      32,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      pacblupdatescreen, //video update routine (called every screen refresh)
      PACBLDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_RIGHT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"superg",      //Game Name
      "Super Galaxians",   //Description
      "galaxian",    //parent
      1,             //no. CPUs
      {{51200,GameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{supergcode,superggfx,galaxcol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      SUPERGALDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"galturbo",      //Game Name
      "Galaxian Turbo",   //Description
      "galaxian",    //parent
      1,             //no. CPUs
      {{51200,GameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{galturbocode,galturbogfx,galaxcol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      SUPERGALDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"galapx",      //Game Name
      "Galaxian Part X",   //Description
      "galaxian",    //parent
      1,             //no. CPUs
      {{51200,GalPXGameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{galpxcode,galpxgfx,galaxcol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      SUPERGALDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"zigzag",      //Game Name
      "ZigZag (Galaxian Hardware)",   //Description
      "zigzag",    //parent
      1,             //no. CPUs
      {{51200,ZigZagRead,ZigZagWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{zigzagcode,zigzaggfx,zigzagcol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      zigzaginputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      pacblgfxs, //gfx decode structure
      256,  //number of characters
      32,   //number of character cols
      64,   //number of sprites
      32,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      zigupdatescreen, //video update routine (called every screen refresh)
      ZIGZAGDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      0,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_RIGHT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initzigzagsnd,
      AY8910_sh_update,0,0
   },
   {"swarm",      //Game Name
      "Swarm",   //Description
      "galaxian",    //parent
      1,             //no. CPUs
      {{51200,GameRead,GameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{swarmcode,swarmgfx,swarmcol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      SWARMDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,  //mutually exclusive inputs
      0
   },
   {"uniwars",      //Game Name
      "UniWar S",   //Description
      "uniwars",    //parent
      1,             //no. CPUs
      {{51200,GameRead,UniGameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{uniwarscode,uniwarsgfx,uniwarscol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      galinputs,  //array of inputs
      0, //screen pointer, allocated later
      inituni, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      unigfxs, //gfx decode structure
      512,  //number of characters
      16,   //number of character cols
      128,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      uniupdatescreen, //video update routine (called every screen refresh)
      SUPERGALDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"hotshock",      //Game Name
      "HotShocker",   //Description
      "hotshock",    //parent
      2,             //no. CPUs
      {
         {51200,HotshockRead,HotshockWrite,ReadPorts,WritePorts,scramint,1},
         {29829,ScrambleSndRead,ScrambleSndWrite,HotshockSndReadPorts,HotshockSndWritePorts,0,1}
      },
      {
         {hotshockcode,hotshockgfx,hotshockcol,0},
         {hotshocksndcode,0,0,0},
      }, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      hotshockinputs,  //array of inputs
      0, //screen pointer, allocated later
      inithotshock, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      unigfxs, //gfx decode structure
      512,  //number of characters
      16,   //number of character cols
      128,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatehotshockscreen, //video update routine (called every screen refresh)
      HOTSHOCKDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      scrampre,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initscramsnd,
      AY8910_sh_update,
      0,0
   },
   {"jumpbug",      //Game Name
      "JumpBug",   //Description
      "jumpbug",    //parent
      1,             //no. CPUs
      {{51200,JumpBugRead,JumpBugWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{jumpbugcode,jumpbuggfx,jumpbugcol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      jumpbuginputs,  //array of inputs
      0, //screen pointer, allocated later
      initjumpbug, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      jumpbuggfxs, //gfx decode structure
      1024,  //number of characters
      16,   //number of character cols
      256,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      jumpbugupdatescreen, //video update routine (called every screen refresh)
      JUMPBUGDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initzigzagsnd,
      AY8910_sh_update,0,0
   },
   {"levers",      //Game Name
      "Levers",   //Description
      "levers",    //parent
      1,             //no. CPUs
      {{51200,LeversRead,JumpBugWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{leverscode,leversgfx,leverscol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      leversinputs,  //array of inputs
      0, //screen pointer, allocated later
      initjumpbug, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      jumpbuggfxs, //gfx decode structure
      1024,  //number of characters
      16,   //number of character cols
      256,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      jumpbugupdatescreen, //video update routine (called every screen refresh)
      LEVERSDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initzigzagsnd,
      AY8910_sh_update,0,0
   },
   {"pisces",      //Game Name
      "Pisces",   //Description
      "pisces",    //parent
      1,             //no. CPUs
      {{51200,GameRead,UniGameWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{piscescode,piscesgfx,piscescol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      piscesinputs,  //array of inputs
      0, //screen pointer, allocated later
      inituni, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      unigfxs, //gfx decode structure
      512,  //number of characters
      16,   //number of character cols
      128,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      uniupdatescreen, //video update routine (called every screen refresh)
      GALDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_LEFT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"orbitron",      //Game Name
      "Orbitron",   //Description
      "orbitron",    //parent
      1,             //no. CPUs
      {{51200,OrbitronRead,OrbitronWrite,ReadPorts,WritePorts,galint,1}},
         //MHz/framerate,ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{orbitroncode,orbitrongfx,orbitroncol,0}}, //code gfx and pallette roms
      "galaxian", //sample set name
      {{"shot.wav"},{"death.wav"},{"back.wav"},{""}}, //samples
      1,    //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      genpal,  //storage space for palette
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}}, //screen sizes,offset and modes
      orbitroninputs,  //array of inputs
      0, //screen pointer, allocated later
      initgal, //initiliase game routine
      cleargal,   //clear down game routine
      0,  //decode graphics routine
      galgfxs, //gfx decode structure
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      updatescreen, //video update routine (called every screen refresh)
      ORBITRONDIPS, //dip switch settings
      galreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      0,    //indicates if video card memory is dirty updated
      0,    //indicates if sprites/chars are dirty updated to buffer
      ORIG_ROT_RIGHT, //original orientation of game
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,  //mutually exclusive inputs
      0  // scaler
   },
   {"earthinv",
      "Earth Invaders",
      "earthinv",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{earthinvcode,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initinv,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      EARTHINVDIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"invaders",   //MIDWAY
      "Space Invaders",
      "invaders",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{invcode,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initinv,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INVDIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"spaceph",
      "Space Phantom",
      "spaceph",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{spacephcode,0,0,0}},
      "invaders",
      {{"xx.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      intspacephinv,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INVDIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"lrescue", //TAITO
      "Lunar Rescue",
      "lrescue",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{lrescuecode,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initlr,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INVDIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,    //sound update funtion,
      0,0
   },
   {"galxwars",   //TAITO
      "Galaxy Wars",
      "galxwars",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{galwarcode,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initinv,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INVDIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"invadpt2",   //TAITO
      "Space Invaders Part II",
      "invadpt2",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{invpt2code,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initlr,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INV2DIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,    //sound update funtion
      0,0
   },
   {"ballbomb",   //TAITO
      "Balloon Bomber",
      "ballbomb",
      1,
      {{33333,InvRead,InvWrite,InvReadPorts,InvWritePorts,invint,2}},
      {{ballbombcode,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initlr,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INV2DIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,    //sound update funtion
      0,0
   },
   {"invrvnge",   //???
      "Invaders Revenge",
      "invrvnge",
      1,
      {{33333,InvRead,InvWrite,InvRvReadPorts,InvWritePorts,invint,2}},
      {{invrvngecode,0,0,0}},
      "invaders",
      {{"0.wav"},{"1.wav"},{"2.wav"},{"3.wav"},{"4.wav"},{"5.wav"},{"6.wav"},{"7.wav"},{"8.wav"},{""}},
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      0,    //use streams
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      invinputs,
      0,
      initir,clearinv,
      0,0,0,0,0,0,0,0,0,0,
      0,    //rotate tiles
      invsetpal,0,0,
      invupdatescreen,
      INVDIPS,
      0,
      invscrclear,
      0,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,    //sound update funtion
      0,0
   },
   {"phoenix", //AMSTAR
      "Phoenix",
      "phoenix",
      1,
      {{51200,PhoenixRead,PhoenixWrite,ReadPorts,WritePorts,phoenixint,1}},
      {{phoenixcode,phoenixgfx,phoenixcol,0}},
      "phoenix",
      {{"shot8.wav"},{"death8.wav"},{"phoenix1.wav"},{"phoenix2.wav"},
      {""}},
      0,
      0,
      bigpal,
      {{208,256,0,0,16,0,3},{256,208,0,0,0,12,0}},
      phoenixinputs,
      0,
      initphoenix,clearphoenix,
      0,
      phnxgfxs,
      512,
      16,
      0,
      0,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      phoenixsetpal,0,0,
      phoenixupdatescreen,
      PHNXDIPS,
      phoenixstartsound,
      0,1,0,0,ORIG_ROT_LEFT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,    //sound update funtion
      0,0
   },
   {"dkong",   //NINTENDO (America)
      "Donkey Kong (America)",
      "dkong",
      1,
      {{51200,DKRead,DKWrite,ReadPorts,WritePorts,dkint,1}},
      {{dkongcode,dkonggfx,dkongcol,0}},
      "dkong",
      {{"effect00.wav"},{"effect01.wav"},{"bounce.wav"},
         {"tune1.wav"},{"noise.wav"},{"tune2.wav"},
         {"tune3.wav"},{"tune4.wav"},{"tune5.wav"},
         {"tune6.wav"},{"tune7.wav"},{"end1.wav"},
         {"tune8.wav"},{"fall.wav"},{"hhit.wav"},
         {"death.wav"},{"coin.wav"},{"prize.wav"},
         {""}},
      0,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      dkonginputs,
      0,
      initdk,cleardk,
      dkdecodegfx,
      dkgfxs,
      256,  //number of characters
      64,   //number of colours
      128,   //number of sprites
      64,   //number of colours
      0,    //number of big sprites
      0,
      0,    //number of tiles
      0,
      0,    //rotate tiles
      dksetpal,0,0,
      dkupdatescreen,
      DKONGDIPS,
      0,0,0,1,1,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"dkongjp",   //NINTENDO (Japan)
      "Donkey Kong (Japan)",
      "dkong",
      1,
      {{51200,DKRead,DKWrite,ReadPorts,WritePorts,dkint,1}},
      {{dkongcodejp,dkonggfxjp,dkongcol,0}},
      "dkong",
      {{"effect00.wav"},{"effect01.wav"},{"bounce.wav"},
         {"tune1.wav"},{"noise.wav"},{"tune2.wav"},
         {"tune3.wav"},{"tune4.wav"},{"tune5.wav"},
         {"tune6.wav"},{"tune7.wav"},{"end1.wav"},
         {"tune8.wav"},{"fall.wav"},{"hhit.wav"},
         {"death.wav"},{"coin.wav"},{"prize.wav"},
         {""}},
      0,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      dkonginputs,
      0,
      initdk,cleardk,
      dkdecodegfx,
      dkgfxs,
      256,  //number of characters
      64,   //number of colours
      128,   //number of sprites
      64,   //number of colours
      0,    //number of big sprites
      0,
      0,    //number of tiles
      0,
      0,    //rotate tiles
      dksetpal,0,0,
      dkupdatescreen,
      DKONGDIPS,
      0,0,0,1,1,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"dkongjr",   //NINTENDO (America)
      "Donkey Kong Jnr",
      "dkongjr",
      1,
      {{51200,DKRead,DKJrWrite,ReadPorts,WritePorts,dkint,1}},
      {{dkjrcode,dkjrgfx,dkjrcol,0}},
      "dkongjr",
      {{"jump.wav"},{"land.wav"},{"roar.wav"},{"climb.wav"},
         {"death.wav"},{"drop.wav"},{"walk.wav"},{"coin.wav"},
         {"tune1.wav"},{"tune2.wav"},{"tune3.wav"},
         {"cheep.wav"},{"dcheep.wav"},{"tune4.wav"},
         {"tune5.wav"},{"tune6.wav"},{"tune7.wav"},
         {"tune8.wav"},{"hhit.wav"},{"scream.wav"},
         {"tune9.wav"},
         {""}},
      0,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      dkonginputs,
      0,
      initdk,cleardk,
      dkdecodegfx,
      dkjrgfxs,
      512,  //number of characters
      64,   //number of colours
      128,   //number of sprites
      64,   //number of colours
      0,    //number of big sprites
      0,
      0,    //number of tiles
      0,
      0,    //rotate tiles
      dksetpal,0,0,
      dkjrupdatescreen,
      DKONGDIPS,
      0,0,0,1,1,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,     //sound update funtion
      0,0
   },
   {"dkngjrjp",   //NINTENDO (Japan)
      "Donkey Kong Jnr (Japan)",
      "dkongjr",
      1,
      {{51200,DKRead,DKJrWrite,ReadPorts,WritePorts,dkint,1}},
      {{dkjrcodej,dkjrgfx,dkjrcol,0}},
      "dkongjr",
      {{"jump.wav"},{"land.wav"},{"roar.wav"},{"climb.wav"},
         {"death.wav"},{"drop.wav"},{"walk.wav"},{"coin.wav"},
         {"tune1.wav"},{"tune2.wav"},{"tune3.wav"},
         {"cheep.wav"},{"dcheep.wav"},{"tune4.wav"},
         {"tune5.wav"},{"tune6.wav"},{"tune7.wav"},
         {"tune8.wav"},{"hhit.wav"},{"scream.wav"},
         {"tune9.wav"},
         {""}},
      0,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      dkonginputs,
      0,
      initdk,cleardk,
      dkdecodegfx,
      dkjrgfxs,
      512,  //number of characters
      64,   //number of colours
      128,   //number of sprites
      64,   //number of colours
      0,    //number of big sprites
      0,
      0,    //number of tiles
      0,
      0,    //rotate tiles
      dksetpal,0,0,
      dkjrupdatescreen,
      DKONGDIPS,
      0,0,0,1,1,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,    //sound init function
      0,    //sound update funtion
      0,0
   },
   {"mooncrst",   //Nichibutsu
      "Moon Cresta",
      "mooncrst",
      1,
      {{51200,MCrstRead,MCrstWrite,ReadPorts,WritePorts,mcrstint,1}},
      {{mcrstcode,mcrstgfx,mcrstcol,0}},
      "galaxian",
      {
         {"shot.wav"},{"death.wav"},{""}
      },
      1,
      0,
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      mcrstinputs,
      0,
      initmcrst,clearmcrst,
      0,
      mcgfxs,
      512,  //number of characters
      16,   //number of colours
      128,  //number of sprites
      16,   //number of colours
      0,    //number of big sprites
      0,
      0,    //number of tiles
      0,
      0,    //rotate tiles
      setpal,0,0,
      updatemcrstscreen,
      MCRSTDIPS,
      0,0,1,0,0,ORIG_ROT_LEFT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      galsndinit,    //sound init function
      galsndupd,     //sound update funtion
      0,0
   },
   {"checkman",
      "Checkman",
      "checkman",
      2,
      {
         {51200,CheckManRead,CheckManWrite,ReadPorts,CheckManWritePorts,checkmanint,1},
         {27000,CheckManSndRead,CheckManSndWrite,CheckManSndReadPorts,CheckManSndWritePorts,0,1}
      },
      {
         {checkmancode,checkmangfx,checkmancol,0},
         {checkmansndcode,0,0,0},
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      checkmaninputs,
      0,
      initcheckman,clearmcrst,
      0,
      mcgfxs,
      512,  //number of characters
      16,   //number of colours
      128,  //number of sprites
      16,   //number of colours
      0,    //number of big sprites
      0,
      0,    //number of tiles
      0,
      0,    //rotate tiles
      setpal,0,0,
      updatecheckmanscr,
      CHECKMANDIPS,
      0,0,1,0,1,ORIG_ROT_LEFT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initzigzagsnd,    //sound init function
      AY8910_sh_update,     //sound update funtion
      0,0
   },

   {"1942", //Capcom
      "1942",
      "1942",
      2,
      {
         {66667,_1942Read,_1942Write,ReadPorts,WritePorts,_1942int,2},
         {50000,_1942sndread,_1942sndwrite,ReadPorts,WritePorts,_1942sndint,4}
      },
      {
         {_1942code,_1942gfx,_1942col,0},
         {_1942snd,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,24,0,8,0,3},{256,224,0,16,0,8,0}},
      _1942inputs,
      0,
      init1942,clear1942,
      0,
      _1942gfxs,
      512,  //number of characters
      64,   //number of colours
      512,  //number of sprites
      16,   //number of colours
      0,    //number of big sprites
      0,
      512,  //number of tiles
      1,
      1,    //rotate tiles
      _1942setpal,0,0,
      0,
      _1942DIPS,
      0,0,1,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      _1942sndinit,    //sound init function
      AY8910_sh_update,     //sound update funtion
      0,0
   },
   {"1943", //Capcom
      "1943",
      "1943",
      2,
      {
         {100000,_1943Read,_1943Write,ReadPorts,WritePorts,_1943int,1},
         {50000,_1943sndread,_1943sndwrite,ReadPorts,WritePorts,_1943sndint,4}
      },
      {
         {_1943code,_1943gfx,_1943col,0},
         {_1943snd,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,24,0,8,0,3},{256,224,0,16,0,8,0}},
      _1943inputs,
      0,
      init1943,clear1943,
      _1943decodegfx,
      _1943gfxs, //1943 has too many chars/sprites/tiles for VAntAGE's standard routines - so only 1 colour
      2048,  //number of characters
      1,   //number of colours
      2048,  //number of sprites
      1,   //number of colours
      640,    //number of big sprites (used for tiles, - 32x32)
      1,
      0,  //number of tiles
      0,
      0,    //rotate tiles
      _1943setpal,0,0,
      update1943scr,
      _1943DIPS,
      0,0,1,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      _1943sndinit,    //sound init function
      YM2203_sh_update,     //sound update funtion
      0,0
   },
   {"1943kai", //Capcom
      "1943 Kai",
      "1943kai",
      2,
      {
         {100000,_1943Read,_1943Write,ReadPorts,WritePorts,_1943int,1},
         {50000,_1943sndread,_1943sndwrite,ReadPorts,WritePorts,_1943sndint,4}
      },
      {
         {_1943kaicode,_1943kaigfx,_1943kaicol,0},
         {_1943kaisnd,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,24,0,8,0,3},{256,224,0,16,0,8,0}},
      _1943inputs,
      0,
      init1943,clear1943,
      _1943decodegfx,
      _1943gfxs, //1943 has too many chars/sprites/tiles for VAntAGE's standard routines - so only 1 colour
      2048,  //number of characters
      1,   //number of colours
      2048,  //number of sprites
      1,   //number of colours
      640,    //number of big sprites (used for tiles, - 32x32)
      1,
      0,  //number of tiles
      0,
      0,    //rotate tiles
      _1943setpal,0,0,
      update1943scr,
      _1943DIPS,
      0,0,1,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      _1943sndinit,    //sound init function
      YM2203_sh_update,     //sound update funtion
      0,0
   },
   {"gunsmoke", //Capcom
      "GunSmoke",
      "gunsmoke",
      2,
      {
         {100000,GSmokeRead,GSmokeWrite,ReadPorts,WritePorts,gsmokeint,1},
         {50000,GSmokesndread,GSmokesndwrite,ReadPorts,WritePorts,_1943sndint,4}
      },
      {
         {GSmokecode,GSmokegfx,GSmokecol,0},
         {GSmokesnd,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,24,0,8,0,3},{256,224,0,16,0,8,0}},
      GSmokeinputs,
      0,
      initgsmoke,cleargsmoke,
      gsmokedecodegfx,
      gsmokegfxs, //Gun Smoke has too many chars/sprites/tiles for VAntAGE's standard routines - so only 1 colour
      1024,  //number of characters
      1,   //number of colours
      2048,  //number of sprites
      1,   //number of colours
      512,    //number of big sprites (used for tiles, - 32x32)
      1,
      0,  //number of tiles
      0,
      0,    //rotate tiles
      gsmokesetpal,0,0,
      updategsmokescr,
      GSMOKEDIPS,
      0,0,1,0,0,ORIG_ROT_RIGHT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      _1943sndinit,    //sound init function
      YM2203_sh_update,     //sound update funtion
      0,0
   },

   {"bombjack",   //Tehkan
      "Bomb Jack",
      "bombjack",
      2,
      {
         {66667,BJRead,BJWrite,ReadPorts,WritePorts,bjint,1},
         {51200,BJSndRead,BJSndWrite,BJSndReadPorts,BJSndWritePorts,bjsndint,1}
      },
      {
         {bjcode,bjgfx,bjcol,0},
         {bjsnd,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,8,0,8,0,3},{256,224,0,16,0,8,0}},
      bjinputs,
      0,
      initbj,clearbj,
      bjdecodegfx,
      bjgfxs,
      512,  //number of characters
      16,   //number of colours
      128,  //number of sprites
      16,   //number of colours
      32,   //number of big sprites
      16,   //number of colours
      256,  //number of tiles
      16,   //number of colours
      1,    //rotate tiles
      bjsetpal,bjpalupd,0,
      bjupdatescreen,
      BJDIPS,
      0,0,0,1,0,ORIG_ROT_LEFT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initbjsnd,
      AY8910_sh_update,     //sound update funtion
      0,0
   },
   {"zaxxon",   //SEGA
      "Zaxxon",
      "zaxxon",
      1,
      {{66667,zaxRead,zaxWrite,ReadPorts,WritePorts,zaxint,1}},
      {{zaxcode,zaxgfx,zaxcol,0}},
      "zaxxon",
      {{"03.wav"},{"02.wav"},{"01.wav"},{"00.wav"},{"11.wav"},
      {"10.wav"},{"08.wav"},{"23.wav"},{"21.wav"},{"20.wav"},
      {"05.wav"},{"04.wav"},{""}},
      0,
      0,
      genpal,
      {{224,256,8,0,8,0,3},{256,224,0,16,0,0,0}},
      zaxinputs,
      0,
      initzax,clearzax,
      zaxdecodegfx,
      zaxgfxs,
      256,  //number of characters
      32,   //number of colours
      0,    //number of sprites
      0,    //number of colours
      64,   //number of big sprites
      32,   //number of colours
      512, //number of tiles
      2,    //number of colours
      0,    //rotate tiles
      zaxsetpal,0,precalczaxback,
      updatezaxscreen,
      ZAXDIPS,
      0,0,0,0,0,ORIG_ROT_LEFT,
      0,    //function to call before context swap
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      0,
      0,     //sound update funtion
      0,0
   },
   {"frogger",  //Sega
      "Frogger",
      "frogger",
      2,
      {
         {51200,FrogRead,FrogWrite,ReadPorts,WritePorts,frogint,1},
         {29830,FrogRead,FrogSndWrite,FrogReadPorts,FrogWritePorts,0,1}
      },
      {
         {frogcode,froggfx,frogcol,0},
         {frogsndcode,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      genpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      froginputs,
      0,
      initfrog,clearfrog,
      frogdecodegfx,
      froggfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      setpal,0,0,
      updatefrogscreen,
      FROGDIPS,
      0,0,0,0,0,ORIG_ROT_LEFT,
      scrampre,
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initfrogsnd,
      AY8910_sh_update,     //sound update funtion
      0,0
   },
   {"pooyan",  //Konami
      "Pooyan",
      "pooyan",
      2,
      {
         {51200,PooyanRead,PooyanWrite,ReadPorts,WritePorts,pooyanint,1},
         {29830,PooyanReadSound,PooyanWriteSound,ReadPorts,WritePorts,0,1}
      },
      {
         {pooyancode,pooyangfx,pooyancol,0},
         {pooyansndcode,0,0,0}
      },
      "",
      {{""}},
      1,
      0,
      bigpal,
      {{224,256,23,0,8,0,3},{256,224,0,16,0,8,0}},
      pooyaninputs,
      0,
      initpooyan,
      clearpooyan,
      0,
      pyangfxs,
      256,  //number of characters
      16,   //number of character cols
      64,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      pooyansetpal,
      0,
      0,
      pyanupdatescreen,
      PYANDIPS,
      0,    //reset
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      1,    //indicates if games has a background which can be switched on/off
      1,    //indicates if video card memory is dirty updated
      1,
      ORIG_ROT_LEFT, //original orientation of game
      scrampre,
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initpyansnd,
      AY8910_sh_update,     //sound update funtion
      0,0
   },
   {"mrdo",      //Game Name
      "Mr Do",   //Description
      "mrdo",    //parent
      1,             //no. CPUs
      {{66667,MrDoRead,MrDoWrite,ReadPorts,WritePorts,pacint,1}},
         //ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{mrdocode,mrdogfx,mrdocol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1, //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      bigpal,  //storage space for palette
      {{192,256,32,8,16,0,3},{256,192,0,32,0,24,0}}, //screen sizes,offset and modes
      mrdoinputs,  //array of inputs
      0, //screen pointer, allocated later
      initmrdo, //initiliase game routine
      clearmrdo,   //clear down game routine
      0,  //decode graphics routine (for non-standard decodes)
      mrdogfxs,   //graphics description
      1024,  //number of characters
      64,   //number of character cols
      128,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      mrdosetpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      mrdoupdatescreen, //video update routine (called every screen refresh)
      MRDODIPS, //dip switch settings
      mrdoreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      0,    //indicates if games has a background which can be switched on/off
      1,    //indicates if video card memory is dirty updated
      1,
      ORIG_ROT_RIGHT, //original orientation of game
      0,
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initmrdosnd,    //sound init function
      SN76496_sh_update,     //sound update funtion
      0,       //mutex keys
      0
   },

   {"ladybug",      //Game Name
      "LadyBug",   //Description
      "ladybug",    //parent
      1,             //no. CPUs
      {{66667,LadybugRead,LadybugWrite,ReadPorts,WritePorts,ladybugint,1}},
         //ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{ladybugcode,ladybuggfx,ladybugcol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1, //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      bigpal,  //storage space for palette
      {{192,240,39,16,16,8,3},
      {240,192,16,40,0,24,0}}, //screen sizes,offset and modes
      ladybuginputs,  //array of inputs
      0, //screen pointer, allocated later
      initladybug, //initiliase game routine
      clearladybug,   //clear down game routine
      0,  //decode graphics routine (for non-standard decodes)
      ladybuggfxs,   //graphics description
      1024,  //number of characters
      16,   //number of character cols
      128,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      ladybugsetpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      ladybugupdscreen, //video update routine (called every screen refresh)
      LADYBUGDIPS, //dip switch settings
      ladybugreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      0,    //indicates if games has a background which can be switched on/off
      1,    //indicates if video card memory is dirty updated
      1,
      ORIG_ROT_RIGHT, //original orientation of game
      0,
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initladybugsnd,    //sound init function
      SN76496_sh_update,     //sound update funtion
      0,       //mutex keys
      0
   },

   {"yankeedo",      //Game Name
      "Yankee Do",   //Description
      "mrdo",    //parent
      1,             //no. CPUs
      {{66667,MrDoRead,MrDoWrite,ReadPorts,WritePorts,pacint,1}},
         //ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {{yankeedocode,yankeedogfx,yankeedocol,0}}, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1, //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      bigpal,  //storage space for palette
      {{192,256,32,8,16,0,3},{256,192,0,32,0,24,0}}, //screen sizes,offset and modes
      mrdoinputs,  //array of inputs
      0, //screen pointer, allocated later
      initmrdo, //initiliase game routine
      clearmrdo,   //clear down game routine
      0,  //decode graphics routine (for non-standard decodes)
      mrdogfxs,   //graphics description
      1024,  //number of characters
      64,   //number of character cols
      128,   //number of sprites
      16,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      mrdosetpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      0, //initialise video routine
      mrdoupdatescreen, //video update routine (called every screen refresh)
      MRDODIPS, //dip switch settings
      mrdoreset, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      0,    //indicates if games has a background which can be switched on/off
      1,    //indicates if video card memory is dirty updated
      0,
      ORIG_ROT_RIGHT, //original orientation of game
      0,
      0,    //function to call after context swap
      1,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initmrdosnd,    //sound init function
      SN76496_sh_update,     //sound update funtion
      0,       //mutex keys
      0
   },
   {"docastle",      //Game Name
      "Mr Do's Castle",   //Description
      "docastle",    //parent
      2,             //no. CPUs
      {
         {66667,DoCastleRead,DoCastleWrite,ReadPorts,WritePorts,docastleint,1},
         {66667,DoCastleRead2,DoCastleWrite2,ReadPorts,WritePorts,docastleint,8}
      },
         //ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {
         {docastlecode,docastlegfx,docastlecol,0},
         {docastlecode2,0,0,0},
      }, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1, //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      bigpal,  //storage space for palette
      {{192,256,39,8,24,0,3},{256,192,0,32,0,24,0}}, //screen sizes,offset and modes
      docastleinputs,  //array of inputs
      0, //screen pointer, allocated later
      initdocastle, //initiliase game routine
      cleardocastle,   //clear down game routine
      0,  //decode graphics routine (for non-standard decodes)
      docastlegfxs,   //graphics description
      1024,  //number of characters
      32,   //number of character cols
      256,   //number of sprites
      32,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      docastlesetpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      docastleinitvid, //initialise video routine
      docastleupdscreen, //video update routine (called every screen refresh)
      DOCASTLEDIPS, //dip switch settings
      0, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      0,    //indicates if games has a background which can be switched on/off
      1,    //indicates if video card memory is dirty updated
      1,
      ORIG_ROT_RIGHT, //original orientation of game
      0,
      0,    //function to call after context swap
      2,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initdocastlesnd,    //sound init function
      SN76496_sh_update,     //sound update funtion
      0,       //mutex keys
      0
   },
   {"jjack",      //Game Name
      "Jumping Jack",   //Description
      "jjack",    //parent
      2,             //no. CPUs
      {
         {66667,JJackRead,JJackWrite,ReadPorts,WritePorts,docastleint,1},
         {66667,JJackRead2,JJackWrite2,ReadPorts,WritePorts,docastleint,8}
      },
         //ReadMem,WriteMem,ReadPorts,WritePorts,Interrupt Routine,ints per screen refresh
      {
         {jjackcode,jjackgfx,jjackcol,0},
         {jjackcode2,0,0,0},
      }, //code gfx and pallette roms
      "", //sample set name
      {{""}}, //samples
      1, //use streams
      0,    //interrupt trigger before each CPU exe slice (0=after,1=before)
      bigpal,  //storage space for palette
      {{192,256,39,8,24,0,3},{256,192,0,32,0,16,0}}, //screen sizes,offset and modes
      docastleinputs,  //array of inputs
      0, //screen pointer, allocated later
      initjjack, //initiliase game routine
      cleardocastle,   //clear down game routine
      0,  //decode graphics routine (for non-standard decodes)
      jjackgfxs,   //graphics description
      1024,  //number of characters
      32,   //number of character cols
      256,   //number of sprites
      32,   //number of sprite cols
      0,    //number of big sprites
      0,    //number of big sprites cols
      0,    //number of tiles
      0,    //number of tiles cols
      0,    //rotate tiles
      docastlesetpal,  //set palette routine
      0, //palette update routine (called every screen refresh)
      jjackinitvid, //initialise video routine
      docastleupdscreen, //video update routine (called every screen refresh)
      JJACKDIPS, //dip switch settings
      0, //called when game is reset by user
      0,    //indicates if UI has to be cleared (partial non-dirty screen updates i.e. invaders)
      0,    //indicates if games has a background which can be switched on/off
      1,    //indicates if video card memory is dirty updated
      1,
      ORIG_ROT_RIGHT, //original orientation of game
      0,
      0,    //function to call after context swap
      2,    //divisor of CPUs, or no cycles to execute on each CPU (if >20)
      initdocastlesnd,    //sound init function
      SN76496_sh_update,     //sound update funtion
      0,       //mutex keys
      0
   },
   {"jrpacman",  //NAMCO
      "Jr Pac-Man",
      "jrpacman",
      1,
      {{51200,JrPacRead,JrPacWrite,ReadPorts,PacWritePorts,pacint,1}},
      {{jrpaccode,jrpacgfx,jrpaccol,jrpacsound}},
      "",
      {{""}},
      0,
      0,
      bigpal,
      {{224,288,23,0,0,0,9},{288,224,0,16,0,0,6}},
      jrpacinputs,
      0,
      initjrpac,clearjrpac,
      0,
      jrpacgfxs,
      512,
      1,
      128,
      1,
      0,
      0,
      0,
      0,
      0,    //rotate tiles
      jrpacsetpal,0,0,
      jrpacupdatescreen,
      JRPACDIPS,
      0,0,0,
      0,
      0,
      ORIG_ROT_LEFT,0,0,
      1,namcostartsound,namcoupdatesound,0,0
   }
};


pBOARD getboard(int argc,char **argv)
{
   int i,j,k;
   int gameno=-1;
   pGAMEINFO pGameInfo;
   int bestmatch=0;
   int closeidx=-1;

   int noboards=sizeof(boards)/sizeof(BOARD);



   strlwr(argv[1]);


   for(i=0;i<noboards;i++)
   {
      int match=0;
      k=0;
      for(j=0;j<strlen(argv[1]);j++)
      {
         if(boards[i].name[k]==argv[1][j])
         {
            match++;
            k++;
         }
      }
      if(match>bestmatch)
      {
         bestmatch=match;
         closeidx=i;
      }
   }



//default to max volume
   vol=15;

   if(argc>1)
   {
//check for list switch
      if(!stricmp(argv[1],"-listgames"))
      {
         banner();
         pGameInfo=(pGAMEINFO)myalloc(noboards*sizeof(GAMEINFO));
         for(i=0;i<noboards;i++)
         {
            strcpy(pGameInfo[i].name,boards[i].name);
            strcpy(pGameInfo[i].game,boards[i].game);
            strcpy(pGameInfo[i].basename,boards[i].basename);
         }
         qsort((void *)pGameInfo,noboards,sizeof(GAMEINFO),gamesort);

         printf("\n%d Games Supported\n\n",noboards);
         printf("----------------------------------------\n");
         for(i=0;i<noboards;i++)
         {
            printf("Description : %s\nGame Name   : %s\n",pGameInfo[i].game,pGameInfo[i].name);
            if(stricmp(pGameInfo[i].name,pGameInfo[i].basename))
               printf("Clone Of    : %s\n",pGameInfo[i].basename);
            printf("----------------------------------------\n");
         }
         printf("========================================\n");
         myfree(pGameInfo);
         return NULL;
      }
      if(!stricmp(argv[1],"-genhtml"))
      {
         FILE *fout;
         FILE *fout2;
         printf("Generating VAntAGE html page (vantage.htm,gamelist.htm)\n");

         fout=fopen("vantage.htm","wt");
         fout2=fopen("gamelist.htm","wt");

         fprintf(fout,"</font><font face=\"Arial\" size=\"2\" color=\"#FFFF00\"><strong>\n");
         fprintf(fout,"<br>Supported Games<br></font></strong>\n");
         fprintf(fout,"<font face=\"Arial\" size=\"2\" >\n");
         fprintf(fout,"<hr size=2>\n");
         fprintf(fout,"<br>\n");
         fprintf(fout,"<table border=\"1\">\n");
         fprintf(fout,"<tr>\n");
         fprintf(fout,"<td><font face=\"Arial\" size=\"2\" >Description&nbsp;</font></td>\n");
         fprintf(fout,"<td><font face=\"Arial\" size=\"2\" >Name&nbsp;</font></td>\n");
         fprintf(fout,"</tr>\n");


         fprintf(fout2,"VAntAGE currently supports:<br>\n");
         fprintf(fout2,"</font><font face=\"Arial\" size=\"2\" color=\"#FFE0\">\n");

         pGameInfo=(pGAMEINFO)myalloc(noboards*sizeof(GAMEINFO));
         for(i=0;i<noboards;i++)
         {
            strcpy(pGameInfo[i].name,boards[i].name);
            strcpy(pGameInfo[i].game,boards[i].game);
         }
         qsort((void *)pGameInfo,noboards,sizeof(GAMEINFO),gamesort);

         for(i=0;i<noboards;i++)
         {
            fprintf(fout,"<tr>\n");
            fprintf(fout,"<td><font face=\"Arial\" size=\"2\" color=\"#FFFF00\">%s&nbsp;</font></td>\n",
                           pGameInfo[i].game);
            fprintf(fout,"<td><font face=\"Arial\" size=\"2\" color=\"#FFFF00\"><center>%s&nbsp;</font></td>\n",
                           pGameInfo[i].name);
            fprintf(fout,"</tr>\n");
            fprintf(fout2,"&nbsp;&nbsp;&nbsp;%s<br>\n",pGameInfo[i].game);
         }
         myfree(pGameInfo);
         fprintf(fout,"</table>\n");

         fprintf(fout2,"</font><font face=\"Arial\" size=\"2\">\n");
         fprintf(fout2,"<br>\n");

         fclose(fout);
         fclose(fout2);
         return NULL;

      }




      if(!stricmp(argv[1],"-listlong"))
      {
         banner();
         pGameInfo=(pGAMEINFO)myalloc(noboards*sizeof(GAMEINFO));
         for(i=0;i<noboards;i++)
         {
            strcpy(pGameInfo[i].name,boards[i].name);
            strcpy(pGameInfo[i].game,boards[i].game);
            strcpy(pGameInfo[i].basename,boards[i].basename);
            pGameInfo[i].index=i;
         }
         qsort((void *)pGameInfo,noboards,sizeof(GAMEINFO),gamesort);

         printf("\n%d Games Supported\n\n",noboards);

         for(i=0;i<noboards;i++)
         {
            int j;
            printf("----------------------------------------\n");
            printf("Description : %s\n",pGameInfo[i].game);
            printf("----------------------------------------\n");
            printf("Game Name   : %s\n",pGameInfo[i].name);
            if(stricmp(pGameInfo[i].name,pGameInfo[i].basename))
               printf("Clone Of    : %s\n",pGameInfo[i].basename);
            printf("Number of CPUs :%d\n",boards[pGameInfo[i].index].nocpus);
            for(j=0;j<boards[pGameInfo[i].index].nocpus;j++)
            {
               listroms("Code ROM",j,boards[pGameInfo[i].index].boardROM[j].coderoms);
               listroms("GFX ROM",j,boards[pGameInfo[i].index].boardROM[j].gfxroms);
               listroms("Pal ROM",j,boards[pGameInfo[i].index].boardROM[j].palroms);
               listroms("Audio ROM",j,boards[pGameInfo[i].index].boardROM[j].soundroms);
            }
            listsamples(boards[pGameInfo[i].index].boardSample,boards[pGameInfo[i].index].samplename);
         //   printf("----------------------------------------\n");
         }
         printf("========================================\n");
         myfree(pGameInfo);
         return NULL;
      }

      if(!stricmp(argv[1],"-genbat"))
      {
         pGameInfo=(pGAMEINFO)myalloc(noboards*sizeof(GAMEINFO));
         for(i=0;i<noboards;i++)
         {
            strcpy(pGameInfo[i].name,boards[i].name);
            strcpy(pGameInfo[i].game,boards[i].game);
            strcpy(pGameInfo[i].basename,boards[i].basename);
         }
         qsort((void *)pGameInfo,noboards,sizeof(GAMEINFO),gamesortshort);

         printf("@echo off\n");
         printf("rem VAntAGE test batch file\n\n");
         for(i=0;i<noboards;i++)
         {
            printf("echo Running %s (ROM Name:%s) %d of %d\n",pGameInfo[i].game,pGameInfo[i].name,i+1,noboards);
            printf("vantage %s\n",pGameInfo[i].name);
         }
         myfree(pGameInfo);

         return NULL;
      }
      if(!stricmp(argv[1],"-listshort"))
      {
         banner();
         pGameInfo=(pGAMEINFO)myalloc(noboards*sizeof(GAMEINFO));
         for(i=0;i<noboards;i++)
         {
            strcpy(pGameInfo[i].name,boards[i].name);
            strcpy(pGameInfo[i].game,boards[i].game);
            strcpy(pGameInfo[i].basename,boards[i].basename);
         }
         qsort((void *)pGameInfo,noboards,sizeof(GAMEINFO),gamesortshort);

         printf("%d Games Supported\n",noboards);
         printf("----------------------------------------\n");
         for(i=0;i<noboards;i++)
         {
            printf("%s",pGameInfo[i].name);
            if(i&&!(i%5))
               printf("\n");
            else
            {
               if(i!=(noboards-1))
                  printf(",");
            }
         }
         printf("\n");
         printf("========================================\n");
         myfree(pGameInfo);
         return NULL;
      }

      if(!stricmp(argv[1],"-genaos"))
      {
         FILE *fout;
         FILE *fout2;

         banner();
         printf("Generate VAntAGE support files for ArcadeOS\n");
         genaosconfiles();

         fout=fopen("vantage.nms","wt");
         fout2=fopen("vantage.cln","wt");
         pGameInfo=(pGAMEINFO)myalloc(noboards*sizeof(GAMEINFO));
         for(i=0;i<noboards;i++)
         {
            strcpy(pGameInfo[i].name,boards[i].name);
            strcpy(pGameInfo[i].game,boards[i].game);
            strcpy(pGameInfo[i].basename,boards[i].basename);
         }
         qsort((void *)pGameInfo,noboards,sizeof(GAMEINFO),gamesort);

         for(i=0;i<noboards;i++)
         {
            fprintf(fout,"%s|%s\n",pGameInfo[i].game,pGameInfo[i].name);
            if(stricmp(pGameInfo[i].basename,pGameInfo[i].name))
               fprintf(fout2,"%s|%s\n",pGameInfo[i].name,pGameInfo[i].basename);
         }
         printf("\nFollowing files created (copy to your ArcadeOS directory):\n");
         printf("vantage.nms\nvantage.cln\nvantage.app\nvantage.dat\n");
         myfree(pGameInfo);
         fclose(fout);
         fclose(fout2);
         return NULL;
      }

      if(!stricmp(argv[1],"-help")||!stricmp(argv[1],"-?"))
      {
         banner();
         disphelp();
         return NULL;
      }

      setoptions(argc,argv);

      openlogfile();

      if(vol<0)
         vol=0;
      if(vol>=NO_VOL_STEPS)
         vol=NO_VOL_STEPS-1;
      for(i=0;i<noboards;i++)
      {
         if(!stricmp(argv[1],boards[i].name))
         {
            gameno=i;
            break;
         }
      }
   }

   if(gameno<0)
   {
      banner();
      if(argv[1])
      {
         printf("\nUnsupported game : %s\n",argv[1]);
         loginfo("Unsupported game : %s",argv[1]);
      }
      if(closeidx>=0&&(argv[1]&&*argv[1]))
      {
         printf("Nearest match %s (%s)\n",boards[closeidx].name,boards[closeidx].game);
         loginfo("Nearest match %s (%s)",boards[closeidx].name,boards[closeidx].game);
         gameno=closeidx;
      }
   }

   if(gameno<0)
   {
      printf("\nUsage:\nvantage <game name> [-option] [-option] ....\n");
      printf("\nvantage -listgames\nfor list of supported games\n");
      printf("\nvantage -listshort\nfor short form list of supported games\n");
      printf("\nvantage -help\nfor command line options\n\n");
      return NULL;
   }
   BoardIdx=gameno;
   setrotations();
   return &boards[gameno];
}


void banner(void)
{
   printf("V.Ant.A.G.E. - Vertical ANTiquated Arcade Game Emulator - Version 1.1\n");
   printf("Brian Lewis 6th Feb 2001\n");
}

void setoptions(int argc,char **argv)
{
   int idx,cidx;
   static char buffer[256];
//then override with any command line arguments
   if(argc>2)
   {
      for(idx=2;idx<argc;idx++)
      {
         cidx=0;
         while(configs[cidx].pvalue)
         {
            sprintf(buffer,"-%s",configs[cidx].name);
            if(!stricmp(argv[idx],buffer))
            {
               *configs[cidx].pvalue=1;
               if(configs[cidx].overidevalue)
                  *configs[cidx].overidevalue=0;
               configs[cidx].setbycommand=TRUE;
            }
            sprintf(buffer,"-no%s",configs[cidx].name);
            if(!stricmp(argv[idx],buffer))
            {
               *configs[cidx].pvalue=0;
               configs[cidx].setbycommand=TRUE;
            }

            cidx++;
         }

         cidx=0;
         while(configvals[cidx].pvalue)
         {
            sprintf(buffer,"-%s",configvals[cidx].name);
            if(!stricmp(argv[idx],buffer))
            {
               if((idx+1)<argc)
               {
                  *configvals[cidx].pvalue=atoi(argv[idx+1]);
                  if(configvals[cidx].overidevalue)
                     *configvals[cidx].overidevalue=0;
                  configs[cidx].setbycommand=TRUE;
               }

            }
            cidx++;
         }
      }
   }
}

void setrotations(void)
{
   if(origrotate)
   {
      if(boards[BoardIdx].origrot==ORIG_ROT_LEFT)
      {
         rol=1;
         ror=0;
      }
      else
      {
         ror=1;
         rol=0;
      }
   }

   if(rol||ror)
   {
      scrolling=0;
      realorient=1;
   }

   if(flipxy)
   {
      if(realorient)
      {
         if(rol)
         {
            rol=0;
            ror=1;
         }
         else
         {
            if(ror)
            {
               ror=0;
               rol=1;
            }
         }
         flipxy=0;
      }
      else
         FlipScreen=1;
   }
   if(arcade)
      scanlines=0;

}
int gamesort(const void *a,const void *b)
{
   pGAMEINFO pa;
   pGAMEINFO pb;
   pa=(pGAMEINFO)a;
   pb=(pGAMEINFO)b;
   return stricmp(pa->game,pb->game);
}

int gamesortshort(const void *a,const void *b)
{
   pGAMEINFO pa;
   pGAMEINFO pb;
   pa=(pGAMEINFO)a;
   pb=(pGAMEINFO)b;
   return stricmp(pa->name,pb->name);
}

void saveoptions(void)
{
   writecfgfile(configs,configvals,textvals);
}

void disphelp(void)
{
   int cidx;
   cidx=0;
   printf("Command Line Options:\n-----------------------------------\n");
   while(configs[cidx].pvalue)
   {
      printf("-%s/-no%s : %s\n",configs[cidx].name,configs[cidx].name,configs[cidx].comment);
      cidx++;
   }
   cidx=0;
   while(configvals[cidx].pvalue)
   {
      printf("-%s <val>  : %s\n",configvals[cidx].name,configvals[cidx].comment);
      cidx++;
   }
   printf("-----------------------------------\n");
}

void getfileopts(void)
{
   readcfgfile(configs,configvals,textvals);
}

void markthorttle(void)
{
   configs[7].setbycommand=1;
}

void listroms(char *pTitle,int CPU,pROMSTRUCT pROMS)
{
   int i=0;
   static char plast;
   char *last=&plast;

   if(!pROMS)
      return;

   while(pROMS[i].size)
   {
      if(strcmp(last,pROMS[i].filename))
         printf("(%s CPU:%d)\t\t%s\n",pTitle,CPU+1,pROMS[i].filename);
      last=pROMS[i].filename;
      i++;
   }
}

void listsamples(pBOARDSAMPLE pSample,char *pSampleSet)
{
   int i=0;
   while(*pSample[i].sample)
   {
      if(!i)
         printf(">>  Sample set:\t%s <<\n",pSampleSet);
      printf("(Sample)\t%s\n",pSample[i].sample);
      i++;
   }
}

void genaosconfiles(void)
{
   genaosfile("vantage.dat",datinfo);
   genaosfile("vantage.app",appinfo);
}


void genaosfile(char *name,char **info)
{
   FILE *fout;
   int idx=0;

   fout=fopen(name,"wt");
   while(*info[idx])
      fprintf(fout,"%s\n",info[idx++]);

   fclose(fout);
}


